﻿Imports System.Drawing

Public Class RGB
    ' All values are between 0 and 255.

#Region " Property Procedures "

    Private BValue As Integer
    Public Property B() As Integer
        Get
            Return GetValueWithinRange(BValue)
        End Get
        Set(ByVal value As Integer)
            BValue = value
        End Set
    End Property

    Private GValue As Integer
    Public Property G() As Integer
        Get
            Return GetValueWithinRange(GValue)
        End Get
        Set(ByVal value As Integer)
            GValue = value
        End Set
    End Property

    Private RValue As Integer
    Public Property R() As Integer
        Get
            Return GetValueWithinRange(RValue)
        End Get
        Set(ByVal value As Integer)
            RValue = value
        End Set
    End Property

    Private Function GetValueWithinRange(ByVal value As Integer) As Integer
        Dim local As Integer = value
        local = Math.Min(255, local)
        local = Math.Max(0, local)
        Return local
    End Function

#End Region

    Public Sub New( _
     ByVal R As Integer, ByVal G As Integer, ByVal B As Integer)
        Me.R = R
        Me.G = G
        Me.B = B
    End Sub

    Public Sub New(ByVal value As Color)
        R = value.R
        G = value.G
        B = value.B
    End Sub

    Public Overrides Function ToString() As String
        Return String.Format("({0}, {1}, {2})", R, G, B)
    End Function

#Region " Overloaded Operators "

    Public Overloads Shared Operator +( _
     ByVal value1 As RGB, ByVal value2 As RGB) As RGB
        If value1 Is Nothing OrElse value2 Is Nothing Then
            Return Nothing
        Else
            ' Add two RGB values together.
            Dim newValue As New RGB( _
             value1.RValue + value2.RValue, _
             value1.GValue + value2.GValue, _
             value1.BValue + value2.BValue)
            Return newValue
        End If
    End Operator

    Public Overloads Shared Operator +( _
     ByVal RGBvalue As RGB, ByVal integerValue As Integer) As RGB
        ' Add a single value to an RGB value.
        If RGBvalue Is Nothing Then
            Return Nothing
        Else
            Return New RGB( _
             RGBvalue.RValue + integerValue, _
             RGBvalue.GValue + integerValue, _
             RGBvalue.BValue + integerValue)
        End If
    End Operator

    Public Overloads Shared Operator /( _
     ByVal value As RGB, ByVal divisor As Integer) As RGB
        ' Perform integer division.
        Return value \ divisor
    End Operator

    Public Overloads Shared Operator \( _
     ByVal value As RGB, ByVal divisor As Integer) As RGB
        ' Divide an RGB value by an integer. The results will
        ' always be integers.
        If value Is Nothing Then
            Return Nothing
        ElseIf divisor = 0 Then
            Throw New DivideByZeroException()
        Else
            Return New RGB(value.RValue \ divisor, _
             value.GValue \ divisor, _
             value.BValue \ divisor)
        End If
    End Operator

    Public Overloads Shared Widening Operator _
     CType(ByVal value As RGB) As Drawing.Color
        ' Allow implicit conversion from RGB to Color.
        If value Is Nothing Then
            Return Nothing
        Else
            Return Color.FromArgb(value.R, value.G, value.B)
        End If
    End Operator

    Public Overloads Shared Widening Operator _
     CType(ByVal value As Color) As RGB
        ' Allow implicit conversions from color to RGB.
        Return New RGB(value)
    End Operator

    Public Overloads Shared Narrowing Operator _
     CType(ByVal value As RGB) As HSV
        ' Require explicit conversion (using CType)
        ' from RGB to HSV.
        If value Is Nothing Then
            Return Nothing
        Else
            Return ToHSV(value)
        End If
    End Operator

    Public Overloads Shared Operator =( _
     ByVal value1 As RGB, ByVal value2 As RGB) As Boolean
        ' Compare two RGB instances, either of which might be Nothing.

        If value1 Is Nothing And value2 Is Nothing Then
            ' If both values are Nothing, return True--the colors are the same.
            Return True
        ElseIf value1 Is Nothing Then
            ' They're not both Nothing, but Value1 is Nothing.
            ' They can't be equal.
            Return False
        Else
            ' One or the other of the colors might be Nothing, 
            ' otherwise compare their values.
            Return value1.Equals(value2)
        End If
    End Operator

    Public Overloads Shared Operator =( _
     ByVal value1 As RGB, ByVal value2 As Color) As Boolean
        ' Compare an RGB and a Color. 
        If value1 Is Nothing Then
            ' A Color instance can't be null, so if the RGB 
            ' is Nothing, return False.
            Return False
        Else
            Return value1.Equals(value2)
        End If
    End Operator

    Public Overloads Shared Operator =( _
     ByVal value1 As RGB, ByVal value2 As HSV) As Boolean
        ' Compare an RGB and an HSV, either of which can be null.
        If value1 Is Nothing And value2 Is Nothing Then
            Return True
        ElseIf value1 Is Nothing Then
            Return False
        Else
            Return value1.Equals(value2)
        End If
    End Operator

    Public Overloads Shared Operator <>( _
     ByVal value1 As RGB, ByVal value2 As Color) As Boolean
        Return Not (value1 = value2)
    End Operator

    Public Overloads Shared Operator <>( _
     ByVal value1 As RGB, ByVal value2 As RGB) As Boolean
        Return Not (value1 = value2)
    End Operator

    Public Overloads Shared Operator <>( _
     ByVal value1 As RGB, ByVal value2 As HSV) As Boolean
        Return Not (value1 = value2)
    End Operator

#End Region

#Region " Overloaded Methods "

    Public Overloads Function Equals( _
     ByVal value As RGB) As Boolean
        If value Is Nothing Then
            ' If value is Nothing, the comparison must fail--the current
            ' instance cannot be null, so the two can't be equal.
            Return False
        Else
            Return (Me.R = value.R AndAlso _
             Me.G = value.G AndAlso Me.B = value.B)
        End If
    End Function

    Public Overloads Function Equals( _
     ByVal value As Color) As Boolean
        ' The current instance can't be Nothing, and the Color
        ' can't be Nothing, so there's no need to check for
        ' null references.

        ' Use the Equals(RGB) overload, taking advantage
        ' of the implicit conversion from Color to RGB:
        Return Me.Equals(New RGB(value))
    End Function

    Public Overloads Function Equals( _
     ByVal value As HSV) As Boolean
        If value Is Nothing Then
            ' If the HSV value is Nothing, then
            ' this comparison must fail--the current instance
            ' can't be Nothing.
            Return False
        Else
            Return Me.Equals(New RGB(CType(value, Color)))
        End If
    End Function

    Public Overloads Function GetHashCode() As Integer
        ' Need to provide an arbitrary integer
        ' that represents this color. 
        Return CType(Me, System.Drawing.Color).GetHashCode
    End Function

#End Region

    Private Shared Function ToHSV(ByVal RGB As RGB) As HSV
        ' In this function, R, G, and B values must be scaled 
        ' to be between 0 and 1.
        ' HSV.H will be a value between 0 and 360, and 
        ' HSV.S and HSV.V are between 0 and 1.
        ' The code must scale these to be between 0 and 255 for
        ' the purposes of this application.

        Dim min As Decimal
        Dim max As Decimal
        Dim delta As Decimal

        Dim r As Decimal = RGB.R / 255D
        Dim g As Decimal = RGB.G / 255D
        Dim b As Decimal = RGB.B / 255D

        Dim h As Decimal
        Dim s As Decimal
        Dim v As Decimal

        min = Math.Min(Math.Min(r, g), b)
        max = Math.Max(Math.Max(r, g), b)
        v = max
        delta = max - min
        If max = 0 Or delta = 0 Then
            ' R, G, and B must be 0, or all the same.
            ' In this case, S is 0, and H is undefined.
            ' Using H = 0 is as good as any...
            s = 0
            h = 0
        Else
            s = delta / max
            If r = max Then
                ' Between Yellow and Magenta
                h = (g - b) / delta
            ElseIf g = max Then
                ' Between Cyan and Yellow
                h = 2 + (b - r) / delta
            Else
                ' Between Magenta and Cyan
                h = 4 + (r - g) / delta
            End If

        End If
        ' Scale h to be between 0 and 360. 
        ' This may require adding 360, if the value
        ' is negative.
        h *= 60
        If h < 0 Then
            h += 360
        End If

        ' Scale to the requirements of this 
        ' application. All values are between 0 and 255.
        Return New HSV(CInt(h / 360 * 255), CInt(s * 255), CInt(v * 255))
    End Function
End Class
