﻿Public Class frmMain
    Private Const NUMSENSORS As UInteger = 8
    Private Const NUMEMITTERS As UInteger = 8
    Private Const PACKETSIZE As UInteger = ((NUMSENSORS * NUMEMITTERS) * 2) + 2
    Private Const BUFFERSIZE As UInteger = PACKETSIZE * 2

    Dim bufferindex As Integer = 0
    Dim WithEvents Serial As New System.IO.Ports.SerialPort
    Dim ReceiveBuffer(BUFFERSIZE) As Byte
    Dim ReceiveValuesStartIndex As Integer
    Dim ReceiveValuesEndIndex As Integer
    Dim AnalogValues(NUMEMITTERS - 1, NUMSENSORS - 1) As UShort
    Dim ScaledValues(NUMEMITTERS - 1, NUMSENSORS - 1) As Integer
    Dim ZeroValues(NUMEMITTERS - 1, NUMSENSORS - 1) As UShort
    Dim PeakValues(NUMEMITTERS - 1, NUMSENSORS - 1) As UShort
    Dim DroppedPackets As Integer = 0
    Private Sub FormLoad(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Try
            lbSerialPort.Items.AddRange(System.IO.Ports.SerialPort.GetPortNames())
            lbSerialPort.SelectedIndex = 0
            cbSerialSpeed.SelectedIndex = 0
            SerialInit()
            txtDroppedPackets.Text = "0"
            For emitter As Integer = 0 To NUMEMITTERS - 1
                For sensor As Integer = 0 To NUMSENSORS - 1
                    ZeroValues(emitter, sensor) = 0
                    PeakValues(emitter, sensor) = 4095
                    ScaledValues(emitter, sensor) = 0
                Next
            Next
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".FormLoad() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub FormDispose(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Disposed
        Try
            If (Serial.IsOpen) Then
                Serial.ReadExisting()
                Serial.Close()
            End If
            Serial.Dispose()
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".FormDispose() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub SerialInit()
        Try
            Serial.DataBits = 8
            Serial.DiscardNull = False
            Serial.Encoding = System.Text.Encoding.ASCII
            Serial.Handshake = IO.Ports.Handshake.None
            Serial.Parity = IO.Ports.Parity.None
            Serial.StopBits = IO.Ports.StopBits.One
            Serial.ReadTimeout = 500
            Serial.WriteTimeout = 500
            Serial.ReceivedBytesThreshold = 1
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".SerialInit() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub SerialIn(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles serial.DataReceived
        Try
            Dim Done As Boolean
            Do
                If serial.IsOpen Then
                    If serial.BytesToRead > 0 Then
                        AddAndCheckByte(CByte(serial.ReadByte()))
                    Else
                        Done = True
                    End If
                Else
                    Done = True
                End If
            Loop Until Done
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".SerialIn() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub AddAndCheckByte(ByVal byte1 As Byte)
        Try
            Static Dim StartDelimiter As Byte = Microsoft.VisualBasic.Asc("s")
            Static Dim EndDelimiter As Byte = Microsoft.VisualBasic.Asc("f")
            Dim startindex As Integer
            ReceiveBuffer(bufferindex) = byte1
            If byte1 = EndDelimiter Then
                If (bufferindex > PACKETSIZE - 1) Then
                    startindex = CInt(bufferindex - PACKETSIZE + 1)
                    If CInt(ReceiveBuffer(startindex)) = StartDelimiter Then
                        ReceiveValuesStartIndex = startindex + 1
                        ReceiveValuesEndIndex = bufferindex - 1
                        UpdateArray()
                        bufferindex = 0
                    End If
                End If
            End If
            bufferindex += 1
            If bufferindex >= BUFFERSIZE Then
                bufferindex = 0
                DroppedPackets += 1
            End If
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".AddAndCheckByte() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub TimerTick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        Try
            If serial.IsOpen Then
                UpdateText()
                pnlGraph.Invalidate()
            End If
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".TimerTick() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub UpdateText()
        Try
            txtBuffer.Text = ""
            For index As Integer = 0 To bufferindex
                txtBuffer.Text += ReceiveBuffer(index).ToString + " "
            Next

            txtValues.Text = ""
            For emitter As Integer = 0 To NUMEMITTERS - 1
                For sensor As Integer = 0 To NUMSENSORS - 1
                    Dim newvalue As String = AnalogValues(emitter, sensor).ToString + " "
                    txtValues.Text += newvalue
                    For i As Integer = 0 To (5 - newvalue.Length)
                        txtValues.Text += " "
                    Next
                Next
                txtValues.Text += vbCrLf
            Next
            txtDroppedPackets.Text = DroppedPackets.ToString
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".UpdateText() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub UpdateArray()
        Try
            Dim emitter, sensor, divisor As Integer
            For emitter = 0 To NUMEMITTERS - 1
                For sensor = 0 To NUMSENSORS - 1
                    AnalogValues(emitter, sensor) = CUShort(CUShort(ReceiveBuffer(CInt(emitter * 16 + sensor * 2 + ReceiveValuesStartIndex))) + CUShort(ReceiveBuffer(CInt(emitter * 16 + sensor * 2 + 1 + ReceiveValuesStartIndex))) * 256)
                    divisor = (CInt(PeakValues(emitter, sensor)) - CInt(ZeroValues(emitter, sensor)))
                    If divisor = 0 Then divisor = 1
                    ScaledValues(emitter, sensor) = CInt((CInt(AnalogValues(emitter, sensor)) - CInt(ZeroValues(emitter, sensor))) * 4095 / divisor)
                    If ScaledValues(emitter, sensor) < 0 Then ScaledValues(emitter, sensor) = 0
                    If ScaledValues(emitter, sensor) > 4095 Then ScaledValues(emitter, sensor) = 4095
                Next
            Next
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".UpdateArray() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub UpdateButtons()
        Try
            If serial.IsOpen Then
                btnConnect.Text = "disconnect"
            Else
                btnConnect.Text = "connect"
            End If
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".UpdateButtons() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub Connect(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConnect.Click
        Try
            If Serial.IsOpen Then
                Timer1.Enabled = False
                Serial.ReadExisting()
                Serial.Close()
            Else
                Serial.Open()
                Timer1.Enabled = True
            End If
            UpdateButtons()
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".Connect() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub PortSelect(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lbSerialPort.SelectedIndexChanged
        Try
            Serial.PortName = lbSerialPort.SelectedItem.ToString
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".PortSelect() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub SpeedSelect(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbSerialSpeed.SelectedIndexChanged
        Try
            Serial.BaudRate = CInt(cbSerialSpeed.SelectedItem)
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".SpeedSelect() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Public Shadows Sub Paint() Handles pnlGraph.Paint
        Try
            Static Dim emitterpos(,) As Integer = {{0, 10}, {0, 30}, {0, 50}, {0, 70}, {10, 80}, {30, 80}, {50, 80}, {70, 80}}
            Static Dim sensorpos(,) As Integer = {{80, 10}, {80, 30}, {80, 50}, {80, 70}, {10, 0}, {30, 0}, {50, 0}, {70, 0}}
            Static Dim g1 As Graphics = pnlGraph.CreateGraphics
            Static Dim Pen1 As New Pen(System.Drawing.Brushes.Black)
            Static Dim HSVColor As New HSV(0, 255, 255)
            Static Dim xe, ye, xs, ys As Integer
            Pen1.Color = System.Drawing.Color.FromArgb(255, 255, 0, 0)
            Pen1.Width = 2
            For emitter As Integer = 0 To 3
                For sensor As Integer = 0 To 3
                    xe = emitterpos(emitter, 0)
                    ye = emitterpos(emitter, 1)
                    xs = sensorpos(sensor, 0)
                    ys = sensorpos(sensor, 1)
                    HSVColor.H = 180 - CInt((CUShort(ScaledValues(emitter, sensor)) * 180 / 4096))
                    Pen1.Color = CType(HSVColor, Color)
                    g1.DrawLine(Pen1, xe * 3, 240 - ye * 3, xs * 3, 240 - ys * 3)
                Next
            Next
            For emitter As Integer = 4 To 7
                For sensor As Integer = 4 To 7
                    xe = emitterpos(emitter, 0)
                    ye = emitterpos(emitter, 1)
                    xs = sensorpos(sensor, 0)
                    ys = sensorpos(sensor, 1)
                    HSVColor.H = 180 - CInt((CUShort(ScaledValues(emitter, sensor)) * 180 / 4096))
                    Pen1.Color = CType(HSVColor, Color)
                    g1.DrawLine(Pen1, xe * 3, 240 - ye * 3, xs * 3, 240 - ys * 3)
                Next
            Next
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".Paint() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub Zero(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZero.Click
        Try
            System.Array.Copy(AnalogValues, ZeroValues, AnalogValues.Length)
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".Zero() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub ScaleX(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnScaleX.Click
        Try
            For emitter As Integer = 4 To 7
                For sensor As Integer = 4 To 7
                    PeakValues(emitter, sensor) = AnalogValues(emitter, sensor)
                Next
            Next
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".ScaleX() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
    Private Sub ScaleY(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnScaleY.Click
        Try
            For emitter As Integer = 0 To 3
                For sensor As Integer = 0 To 3
                    PeakValues(emitter, sensor) = AnalogValues(emitter, sensor)
                Next
            Next
        Catch ex As Exception
            MessageBox.Show(Me.Name + ".ScaleY() " + ex.Message, "Exception", MessageBoxButtons.OK)
        End Try
    End Sub
End Class